/*=========================================================================

 Program:   vv
 Module:    $RCSfile: vvConvertITKImageToVTKDataFilter.h,v $
 Language:  C++
 Date:      $Date: 2008/06/03 11:58:37 $
 Version:   $Revision: 1.2 $
 Author :   Pierre Seroul (pierre.seroul@gmail.com)

Copyright (C) 2008
Léon Bérard cancer center http://oncora1.lyon.fnclcc.fr
CREATIS-LRMN http://www.creatis.insa-lyon.fr

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=========================================================================*/
#ifndef vvConvertITKImageToVTKDataFilter_H
#define vvConvertITKImageToVTKDataFilter_H

#include <string.h>

// ITK includes
#include "itkImage.h"
#include "itkImageFileReader.h"
#include "itkImageSeriesReader.h"
#include "itkImageToVTKImageFilter.h"
#include "itkExtractImageFilter.h"
#include "itkCommand.h"
#include "vtkImageData.h"
#include "vvImage.h"

class vvConvertITKImageToVTKDataFilter {
    
  public: 
    // constructor
    vvConvertITKImageToVTKDataFilter();
    ~vvConvertITKImageToVTKDataFilter();

    void SetInputFileName(const std::vector<std::string> & filename);
    void SetObserver(itk::Command::Pointer o) { mUseAnObserver = true; mObserver = o; }
    void DisableObserver() { mUseAnObserver = false; }

    vvImage* GetOutput(){return mImage;}

    std::string GetLastError() {return mLastError;}

    //====================================================================
    // Main function
    void Update(int dim, std::string InputPixelType, int type);
    //void Extract(int dim, std::string InputPixelType, int slice);

  protected:
    //====================================================================
    std::vector<std::string> mInputFilename;
    int mType;
    itk::Command::Pointer mObserver;

    std::string mLastError;
    bool mUseAnObserver;

    //====================================================================
    template<unsigned int VImageDimension>
    void UpdateWithDim(std::string inputPixelType);

    //====================================================================
    /*template<unsigned int VImageDimension>
    void ExtractWithDim(std::string inputPixelType, int slice);*/

    //====================================================================
    template<class InputPixelType, unsigned int VImageDimension>
    void UpdateWithDimAndInputPixelType();

    //====================================================================
    template<class InputPixelType, unsigned int VImageDimension>
    void UpdateForDifferentDims();

    //====================================================================
    template<class InputPixelType, unsigned int VImageDimension>
    void UpdateForSameDims();

    //====================================================================
    /*template<class InputPixelType, unsigned int VImageDimension>
    void ExtractOneSlice(int slice);*/

 private:
   vvImage* mImage;

  }; // end class vvConvertITKImageToVTKDataFilter

#include "vvConvertITKImageToVTKDataFilter.txx"

#endif /* end #define CLITKvvConvertITKImageToVTKDataFilter_H */

