/*=========================================================================

 Program:   vv
 Module:    $RCSfile: vvConvertITKVectorToVTKDataFilter.h,v $
 Language:  C++
 Date:      $Date: 2008/05/28 14:17:30 $
 Version:   $Revision: 1.1 $
 Author :   Pierre Seroul (pierre.seroul@gmail.com)

Copyright (C) 2008
Léon Bérard cancer center http://oncora1.lyon.fnclcc.fr
CREATIS-LRMN http://www.creatis.insa-lyon.fr

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=========================================================================*/
#ifndef vvConvertITKVectorToVTKDataFilter_H
#define vvConvertITKVectorToVTKDataFilter_H

#include <string.h>

// ITK includes
#include "itkImage.h"
#include "itkImageFileReader.h"
#include "itkImageSeriesReader.h"
#include "itkImageToVTKImageFilter.h"
#include "itkExtractImageFilter.h"
#include "itkCommand.h"
#include "vtkStructuredGrid.h"
#include "vtkPoints.h"
#include "vtkPointData.h"
#include "vtkFloatArray.h"
#include "vvVF.h"

class vvConvertITKVectorToVTKDataFilter {
    
  public: 
    // constructor
    vvConvertITKVectorToVTKDataFilter();
    ~vvConvertITKVectorToVTKDataFilter();

    void SetInputFileName(const std::string & filename);
    void SetObserver(itk::Command::Pointer o) { mUseAnObserver = true; mObserver = o; }
    void DisableObserver() { mUseAnObserver = false; }

    vvVF* GetOutput(){return mImage;}

    std::string GetLastError() {return mLastError;}

    //====================================================================
    // Main function
    void Update(unsigned int dim, std::string InputPixelType);
    //void Extract(int dim, std::string InputPixelType, int slice);

  protected:
    //====================================================================
    
    std::string mInputFilename;
    itk::Command::Pointer mObserver;

    std::string mLastError;
    bool mUseAnObserver;

    //====================================================================
    template<unsigned int VImageDimension>
    void UpdateWithDim(std::string inputPixelType);

    //====================================================================
    template<class InputPixelType, unsigned int VImageDimension>
    void UpdateWithDimAndInputPixelType();

    //====================================================================
    template<class InputPixelType, unsigned int VImageDimension>
    void UpdateForDifferentDims();

    //====================================================================
    template<class InputPixelType, unsigned int VImageDimension>
    void UpdateForSameDims();

 private:
   vvVF* mImage;

  }; // end class vvConvertITKVectorToVTKDataFilter



#include "vvConvertITKVectorToVTKDataFilter.txx"

#endif /* end #define CLITKvvConvertITKVectorToVTKDataFilter_H */

