/*=========================================================================

Program:   vv
Module:    $RCSfile: vvConvertITKVectorToVTKDataFilter.txx,v $
Language:  C++
Date:      $Date: 2008/07/04 08:16:43 $
Version:   $Revision: 1.6 $
Author :   Pierre Seroul (pierre.seroul@gmail.com)

Copyright (C) 2008
Léon Bérard cancer center http://oncora1.lyon.fnclcc.fr
CREATIS-LRMN http://www.creatis.insa-lyon.fr

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=========================================================================*/
#ifndef vvConvertITKVectorToVTKDataFilter_TXX
#define vvConvertITKVectorToVTKDataFilter_TXX

//====================================================================
template<unsigned int VImageDimension>
void vvConvertITKVectorToVTKDataFilter::UpdateWithDim(std::string InputPixelType) 
{
  //std::cout << "UpdateWithDim (" << InputPixelType << ")" << std::endl;
  if(InputPixelType == "short")
    {
      UpdateWithDimAndInputPixelType<short,VImageDimension>();
    }
  else if (InputPixelType == "unsigned_short")
    {
      UpdateWithDimAndInputPixelType<unsigned short,VImageDimension>();
    }
  else if (InputPixelType == "char")
    {
      UpdateWithDimAndInputPixelType<char,VImageDimension>();
    }
  else if (InputPixelType == "unsigned_char")
    {
      UpdateWithDimAndInputPixelType<unsigned char,VImageDimension>();
    }
  else if (InputPixelType == "int")
    {
      UpdateWithDimAndInputPixelType<int,VImageDimension>();
    }
  else if (InputPixelType == "double")
    {
      UpdateWithDimAndInputPixelType<double,VImageDimension>();
    }
  else if (InputPixelType == "float")
    {
      UpdateWithDimAndInputPixelType<float,VImageDimension>();
    }
  else
    {
      std::cerr << "Error, input pixel type : " << InputPixelType << " unknown !" << std::endl;
    }
}
//====================================================================

//====================================================================
template<class InputPixelType, unsigned int VImageDimension>
void vvConvertITKVectorToVTKDataFilter::UpdateWithDimAndInputPixelType() 
{
  //std::cout << "UpdateWithDimAndInputPixelType()" << std::endl;
  if(VImageDimension == 4)
    {
      UpdateForDifferentDims<InputPixelType, VImageDimension>();
    }
  else
    {
      UpdateForSameDims<InputPixelType, VImageDimension>();
    }
}
//====================================================================

//====================================================================
template<class InputPixelType, unsigned int VImageDimension>
void vvConvertITKVectorToVTKDataFilter::UpdateForDifferentDims() 
{
  //std::cout << "UpdateForDifferentDims()" << std::endl;
  mImage->Init();
  //Create the reader
  typedef itk::Vector<InputPixelType,3> InputVectorType;
  typedef itk::Image< InputVectorType, VImageDimension > InputImageType;
  typedef itk::ImageFileReader<InputImageType> ReaderType;
  typename ReaderType::Pointer reader = ReaderType::New();
  reader->SetFileName(mInputFilename);
  if (mUseAnObserver) {
    reader->AddObserver(itk::ProgressEvent(), mObserver);
  }
  try {
    reader->Update(); 
  }
  catch( itk::ExceptionObject & err ) {
    std::cerr << "Error while reading " << mInputFilename.c_str()
	      << " " << err << std::endl;
    std::stringstream error;
    error << err;
    mLastError = error.str();
    return;
  }

  typedef itk::Image< InputVectorType,  3 >    ConnectorImageType;
  typedef itk::ExtractImageFilter<InputImageType,ConnectorImageType> FilterType;

  typename FilterType::Pointer filter = FilterType::New();

  //extract the 3D slices and put them in a std::vector<vtkImageData*>
  typename InputImageType::RegionType inputRegion = reader->GetOutput()->GetLargestPossibleRegion();
  typename InputImageType::SizeType inputSize = inputRegion.GetSize();

  typename InputImageType::SizeType extractedRegionSize = inputSize;
  typename InputImageType::RegionType extractedRegion;
  extractedRegionSize[3] = 0;
  extractedRegion.SetSize(extractedRegionSize);

  filter->SetInput(reader->GetOutput());

  typename InputImageType::IndexType start = inputRegion.GetIndex();

  typename ConnectorImageType::RegionType outputRegion;
  typename ConnectorImageType::SizeType outputSize;
  typename ConnectorImageType::SpacingType outputSpacing;
  typename ConnectorImageType::PointType outputOrigin;


  for(unsigned int num = 0; num < inputSize[3]; num++) {
    start[3] = num;
    extractedRegion.SetIndex(start);
    filter->SetExtractionRegion(extractedRegion);
    filter->Update();
    vtkStructuredGrid* sgrid = vtkStructuredGrid::New();
    int dims[VImageDimension];
    outputRegion = filter->GetOutput()->GetLargestPossibleRegion();
    outputSize = outputRegion.GetSize();
    outputSpacing = filter->GetOutput()->GetSpacing();
    outputOrigin = filter->GetOutput()->GetOrigin();

    for(unsigned int d = 0; d < VImageDimension; d++)
      dims[d] = outputSize[d];

    sgrid->SetDimensions(dims);

    vtkPoints *points = vtkPoints::New();
    points->Allocate(dims[0]*dims[1]*dims[2]);

    vtkFloatArray *vectors = vtkFloatArray::New();
    vectors->SetNumberOfComponents(3);
    vectors->SetNumberOfTuples(dims[0]*dims[1]*dims[2]);

    float x[3], v[3];
    int k,j,i;
    int offset, kOffset, jOffset;
    typename ConnectorImageType::IndexType index;

    for ( k=0; k<dims[2]; k++)
      {
	x[2] = k*outputSpacing[2] + outputOrigin[2];
	index[2] = k;
	kOffset = k * dims[0] * dims[1];
	for (j=0; j<dims[1]; j++) 
	  {
	    x[1] = j*outputSpacing[1] + outputOrigin[1];
	    index[1] = j;
	    jOffset = j * dims[0];
	    for (i=0; i<dims[0]; i++) 
	      {
		x[0] = i*outputSpacing[0] + outputOrigin[0];
		index[0] = i;
		offset = i + jOffset + kOffset;
		InputVectorType vector = filter->GetOutput()->GetPixel(index);
		v[0] = vector[0];
		v[1] = vector[1];
		v[2] = vector[2];
		points->InsertPoint(offset,x);
		vectors->InsertTuple(offset,v);
	      }
	  }
      }
    sgrid->SetPoints(points);
    points->Delete();
    sgrid->GetPointData()->SetVectors(vectors);
    vectors->Delete();
    mImage->AddImage(sgrid);
  }
  std::vector<double> spacing;
  std::vector<double> origin;
  for(int i = 0; i < 3; i++)
    {
      spacing.push_back(outputSpacing[i]);
      origin.push_back(outputOrigin[i]);
    }
  mImage->SetSpacing(spacing);
  mImage->SetOrigin(origin);
}
//====================================================================

//====================================================================
template<class InputPixelType, unsigned int VImageDimension>
void vvConvertITKVectorToVTKDataFilter::UpdateForSameDims() 
{
  //std::cout << "UpdateForSameDims()" << std::endl;
  mImage->Init();

  //Create the reader
  typedef itk::Vector<InputPixelType,3> InputVectorType;
  typedef itk::Image< InputVectorType, VImageDimension > InputImageType;
  typedef itk::ImageFileReader<InputImageType> ReaderType;
  typename ReaderType::Pointer reader = ReaderType::New();
  reader->SetFileName(mInputFilename);
  if (mUseAnObserver) {
    reader->AddObserver(itk::ProgressEvent(), mObserver);
  }
  try {
    reader->Update(); 
  }
  catch( itk::ExceptionObject & err ) {
    std::cerr << "Error while reading " << mInputFilename.c_str()
	      << " " << err << std::endl;
    std::stringstream error;
    error << err;
    mLastError = error.str();
    return;
  }

  vtkStructuredGrid* sgrid = vtkStructuredGrid::New();
  int dims[VImageDimension];
  typename InputImageType::RegionType inputRegion = reader->GetOutput()->GetLargestPossibleRegion();
  typename InputImageType::SizeType inputSize = inputRegion.GetSize();
  typename InputImageType::SpacingType inputSpacing = reader->GetOutput()->GetSpacing();
  typename InputImageType::PointType inputOrigin = reader->GetOutput()->GetOrigin();

  for(unsigned int d = 0; d < VImageDimension; d++)
    dims[d] = inputSize[d];

  sgrid->SetDimensions(dims);

  vtkPoints *points = vtkPoints::New();
  points->Allocate(dims[0]*dims[1]*dims[2]);

  vtkFloatArray *vectors = vtkFloatArray::New();
  vectors->SetNumberOfComponents(3);
  vectors->SetNumberOfTuples(dims[0]*dims[1]*dims[2]);

  float x[3], v[3];
  int k,j,i;
  int offset, kOffset, jOffset;
  typename InputImageType::IndexType index;

  for ( k=0; k<dims[2]; k++) {
    x[2] = k*inputSpacing[2] + inputOrigin[2];
    index[2] = k;
    kOffset = k * dims[0] * dims[1];
    for (j=0; j<dims[1]; j++) 
      {
	x[1] = j*inputSpacing[1] + inputOrigin[1];
	index[1] = j;
	jOffset = j * dims[0];
	for (i=0; i<dims[0]; i++) 
	  {
	    x[0] = i*inputSpacing[0] + inputOrigin[0];
	    index[0] = i;
	    offset = i + jOffset + kOffset;
	    InputVectorType vector = reader->GetOutput()->GetPixel(index);
	    v[0] = vector[0];
	    v[1] = vector[1];
	    v[2] = vector[2];
	    points->InsertPoint(offset,x);
	    vectors->InsertTuple(offset,v);
	  }
      }
  }
  sgrid->SetPoints(points);
  points->Delete();
  sgrid->GetPointData()->SetVectors(vectors);
  vectors->Delete();
  mImage->AddImage(sgrid);
  std::vector<double> spacing;
  std::vector<double> origin;
  for(int i = 0; i < 3; i++)
    {
      spacing.push_back(inputSpacing[i]);
      origin.push_back(inputOrigin[i]);
    }
  mImage->SetSpacing(spacing);
  mImage->SetOrigin(origin);
}
//====================================================================


#endif /* end #define vvConvertITKVectorToVTKDataFilter_TXX */

