/*=========================================================================

 Program:   vv
 Module:    $RCSfile: vvConvertVTKImageToITKDataFilter.txx,v $
 Language:  C++
 Date:      $Date: 2008/05/28 14:17:31 $
 Version:   $Revision: 1.1 $
 Author :   Pierre Seroul (pierre.seroul@gmail.com)

Copyright (C) 2008
Léon Bérard cancer center http://oncora1.lyon.fnclcc.fr
CREATIS-LRMN http://www.creatis.insa-lyon.fr

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=========================================================================*/
#ifndef vvConvertVTKImageToITKDataFilter_TXX
#define vvConvertVTKImageToITKDataFilter_TXX

//====================================================================
template<unsigned int VImageDimension>
void vvConvertVTKImageToITKDataFilter::UpdateWithDim(std::string OutputPixelType) 
{
  if(OutputPixelType == "short")
  {
  UpdateWithDimAndOutputPixelType<short,VImageDimension>();
  }
  else if (OutputPixelType == "unsigned_short")
  {
  UpdateWithDimAndOutputPixelType<unsigned short,VImageDimension>();
  }
  else if (OutputPixelType == "char")
  {
  UpdateWithDimAndOutputPixelType<char,VImageDimension>();
  }
  else if (OutputPixelType == "unsigned_char")
  {
  UpdateWithDimAndOutputPixelType<unsigned char,VImageDimension>();
  }
  else if (OutputPixelType == "int")
  {
  UpdateWithDimAndOutputPixelType<int,VImageDimension>();
  }
  else if (OutputPixelType == "double")
  {
  UpdateWithDimAndOutputPixelType<double,VImageDimension>();
  }
  else if (OutputPixelType == "float")
  {
  UpdateWithDimAndOutputPixelType<float,VImageDimension>();
  }
  else
  {
  std::cerr << "Error, input pixel type : " << OutputPixelType << " unknown !" << std::endl;
  }
}
//====================================================================

//====================================================================
template<class OutputPixelType, unsigned int VImageDimension>
void vvConvertVTKImageToITKDataFilter::UpdateWithDimAndOutputPixelType() 
{
  if(VImageDimension == 4)
    {
    UpdateForDifferentDims<OutputPixelType, VImageDimension>();
    }
  else
    {
    UpdateForSameDims<OutputPixelType, VImageDimension>();
    }
}
//====================================================================

//====================================================================
template<class OutputPixelType, unsigned int VImageDimension>
void vvConvertVTKImageToITKDataFilter::UpdateForDifferentDims() 
{
  //Create the reader
  typedef itk::Image< OutputPixelType, VImageDimension > OutputImageType;
  typedef itk::ImageFileWriter<OutputImageType> WriterType;
  typename WriterType::Pointer writer = WriterType::New();
  writer->SetFileName(mOutputFilename);

  typedef itk::Image< OutputPixelType,  3 >    ConnectorImageType;
  typedef itk::VTKImageToImageFilter <ConnectorImageType> ConnectorType;
  typedef itk::JoinSeriesImageFilter <ConnectorImageType,OutputImageType> FilterType;


  typename FilterType::Pointer filter = FilterType::New();
  filter->SetOrigin(mImage->GetOrigin()[3]);
  filter->SetSpacing(mImage->GetSpacing()[3]);

  for(int i = 0; i < mImage->GetSize()[3]; i++)
    {
    typename ConnectorType::Pointer connector = ConnectorType::New();
    connector->SetInput(mImage->GetImage()[i]);
    connector->Update();
    filter->PushBackInput(connector->GetOutput());
    }
  writer->SetInput(filter->GetOutput());

  if (mUseAnObserver) {
    writer->AddObserver(itk::ProgressEvent(), mObserver);
  }
  try {
    writer->Update(); 
  }
  catch( itk::ExceptionObject & err ) {
    std::cerr << "Error while reading " << mOutputFilename.c_str()
	      << " " << err << std::endl;
    std::stringstream error;
    error << err;
    mLastError = error.str();
    return;
  }
}
//====================================================================

//====================================================================
template<class OutputPixelType, unsigned int VImageDimension>
void vvConvertVTKImageToITKDataFilter::UpdateForSameDims() 
{
  //Create the writer
  typedef itk::Image< OutputPixelType, VImageDimension > OutputImageType;
  typedef itk::ImageFileWriter<OutputImageType> WriterType;
  typename WriterType::Pointer writer = WriterType::New();
  writer->SetFileName(mOutputFilename);

  typedef itk::Image< OutputPixelType,  VImageDimension >    ConnectorImageType;
  typedef itk::VTKImageToImageFilter <ConnectorImageType> ConnectorType;
  typename ConnectorType::Pointer connector = ConnectorType::New();
  connector->SetInput(mImage->GetImage()[0]);

  connector->Update();
  writer->SetInput(connector->GetOutput());

  if (mUseAnObserver) {
    writer->AddObserver(itk::ProgressEvent(), mObserver);
  }
  try {
    writer->Update(); 
  }
  catch( itk::ExceptionObject & err ) {
    std::cerr << "Error while writing " << mOutputFilename.c_str()
	      << " " << err << std::endl;
    std::stringstream error;
    error << err;
    mLastError = error.str();
    return;
  }
}
//====================================================================

#endif /* end #define vvConvertVTKImageToITKDataFilter_TXX */

