/*=========================================================================

 Program:   vv
 Module:    $RCSfile: vvHelp.h,v $
 Language:  C++
 Date:      $Date: 2008/06/25 12:34:31 $
 Version:   $Revision: 1.2 $
 Author :   Pierre Seroul (pierre.seroul@gmail.com)

Copyright (C) 2008
Léon Bérard cancer center http://oncora1.lyon.fnclcc.fr
CREATIS-LRMN http://www.creatis.insa-lyon.fr

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=========================================================================*/
#ifndef __vvHelp_h
#define __vvHelp_h

#include <iostream>
#include <vector>

#include <vvHelp.h>

#include "vtkActor2D.h"

class vtkTextProperty;
class vtkPolyDataMapper2D;
class vtkImageData;
class vtkFreeTypeUtilities;
class vtkTransform;
class vtkPolyData;
class vtkPoints;
class vtkTexture;

class vvHelp : public vtkActor2D
{
public:
  vtkTypeRevisionMacro(vvHelp,vtkActor2D);
  void PrintSelf(ostream& os, vtkIndent indent);

  static vvHelp *New();

  void ShallowCopy(vtkProp *prop);

  void SetMapper(vtkPolyDataMapper2D *mapper);

  void SetInput(const char *inputString);
  char *GetInput();

  vtkSetVector2Macro(MinimumSize,int);
  vtkGetVector2Macro(MinimumSize,int);

  vtkSetMacro(MaximumLineHeight,float);
  vtkGetMacro(MaximumLineHeight,float);

  vtkSetMacro(ScaledText,int);
  vtkGetMacro(ScaledText,int);
  vtkBooleanMacro(ScaledText,int);

  vtkSetMacro(UseBorderAlign,int);
  vtkGetMacro(UseBorderAlign,int);
  vtkBooleanMacro(UseBorderAlign,int);

  void SetAlignmentPoint(int point);
  int GetAlignmentPoint();
  
  void SetOrientation(float orientation);
  vtkGetMacro(Orientation,float);

  // Description:
  // Set/Get the text property.
  virtual void SetTextProperty(vtkTextProperty *p);
  vtkGetObjectMacro(TextProperty,vtkTextProperty);
  

  virtual void SetNonLinearFontScale(double exponent, int target);


  void SpecifiedToDisplay(double *pos, vtkViewport *vport, int specified);


  void DisplayToSpecified(double *pos, vtkViewport *vport, int specified);

//BTX
  virtual void ReleaseGraphicsResources(vtkWindow *);

  int RenderOpaqueGeometry(vtkViewport* viewport);
  virtual int RenderTranslucentPolygonalGeometry(vtkViewport* ) {return 0;};
  int RenderOverlay(vtkViewport* viewport);

  virtual int HasTranslucentPolygonalGeometry();
//ETX

protected:

  void SetMapper(vtkMapper2D *mapper);

   vvHelp();
  ~vvHelp();

  int     MinimumSize[2];
  float   MaximumLineHeight;
  double  FontScaleExponent;
  double  FontScaleTarget;
  int     ScaledText;
  float   Orientation;
  int     UseBorderAlign;

  vtkTextProperty *TextProperty;
  vtkImageData *ImageData;

  vtkPolyDataMapper2D *PDMapper;
  vtkFreeTypeUtilities *FreeTypeUtilities;
  vtkTimeStamp  BuildTime;
  vtkTransform *Transform;
  int LastSize[2];
  int LastOrigin[2];
  char *Input;
  bool InputRendered;
  double FormerOrientation;

  vtkPolyData* Rectangle;
  vtkPoints*   RectanglePoints;
  vtkTexture *Texture; 
  
  void ComputeRectangle(vtkViewport *viewport); 


  virtual void SetTexture(vtkTexture*);
  vtkGetObjectMacro(Texture,vtkTexture);

private:
  vvHelp(const vvHelp&);  // Not implemented.
  void operator=(const vvHelp&);  // Not implemented.
};


#endif

