/*=========================================================================

Program:   vv
Module:    $RCSfile: vvSlicersManager.cxx,v $
Language:  C++
Date:      $Date: 2008/07/15 11:26:35 $
Version:   $Revision: 1.34 $
Author :   Pierre Seroul (pierre.seroul@gmail.com)

Copyright (C) 2008
Léon Bérard cancer center http://oncora1.lyon.fnclcc.fr
CREATIS-LRMN http://www.creatis.insa-lyon.fr

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=========================================================================*/
#include "vvSlicersManager.h"

#include "vvSlicer.h"
#include "vvImage.h"
#include "vvSlicersManagerCommand.h"
#include "vvInteractorStyleNavigator.h"
#include "vvLandmarks.h"

#include "vtkImageActor.h"
#include "vtkImageData.h"
#include "vtkRenderWindow.h"
#include "vtkRendererCollection.h"
#include "vtkRenderWindowInteractor.h"
#include "vtkImageMapToWindowLevelColors.h"
#include "vtkWindowLevelLookupTable.h"
#include "vtkColorTransferFunction.h"
#include "vtkImageClip.h"

#include <vtksys/SystemTools.hxx>
//----------------------------------------------------------------------------

vvSlicersManager::vvSlicersManager(int numberOfSlicers)
{
  mFileName = "";
  mId = "";
  mVFName = "";
  mOverlayName = "";
  mFusionName = "";
  mVFId = "";
  mVFComponent = "";
  mComponent = "";
  mLastError = "";
  mDim = -1;
  mType = -1;
  mColorMap = 0;
  mPreset = 0;
  mOverlayColor = 130;

  mFusionOpacity = 70;
  mFusionColorMap = 3;
  mFusionWindow = 1000;
  mFusionLevel = 1000;

  mConvertor = NULL;
  mVectorConvertor = NULL;
  mOverlayConvertor = NULL;
  mFusionConvertor = NULL;
  mLandmarks = NULL;
  mLinkedId.resize(0);

  for( int i = 0; i < numberOfSlicers; i++)
    {
      vvSlicer *slicer = vvSlicer::New();
      mSlicers.push_back(slicer);
    }
}

vvSlicersManager::~vvSlicersManager()
{
  for( unsigned int i = 0; i < mSlicers.size(); i++)
    {
      if(mSlicers[i] != NULL)
	mSlicers[i]->Delete();
    }
  if(mConvertor)
    {
      delete mConvertor;
    }
  if(mVectorConvertor)
    {
      delete mVectorConvertor;
    }
  if(mOverlayConvertor)
    {
      delete mOverlayConvertor;
    }
  if(mFusionConvertor)
    {
      delete mFusionConvertor;
    }
  if(mLandmarks)
    delete mLandmarks;
}

bool vvSlicersManager::SetImage(std::string filename,int dim, std::string component,int type)
{
  //std::cout << "vvSlicerManager::SetImage()" << std::endl;
  mFileName = filename;
  mDim = dim;
  mComponent = component;
  mType = type;
  if(mConvertor == NULL)
    mConvertor = new vvConvertITKImageToVTKDataFilter;
  std::vector<std::string> filenames;
  filenames.push_back(filename);
  mConvertor->SetInputFileName(filenames);
  mConvertor->Update(dim,component.c_str(),type);
  if(mConvertor->GetLastError().size() == 0)
    {
      for( unsigned int i = 0; i < mSlicers.size(); i++)
	{
	  mSlicers[i]->SetFileName(vtksys::SystemTools::GetFilenameWithoutLastExtension(filename));
	  mSlicers[i]->SetImage(mConvertor->GetOutput());
	}
    }
  else
    {
      mLastError = mConvertor->GetLastError();
      return false;
    }
  return true;
}

bool vvSlicersManager::SetImages(std::vector<std::string> filenames,int dim, std::string component,int type)
{
  //std::cout << "vvSlicerManager::SetImages()" << std::endl;
  dim++;
  mFileName = "";
  mType = type;
  std::string fileWithoutExtension = "";
  if (type == 21)
    {
      mFileName += "dicom_";
      fileWithoutExtension += "dicom_";
    }
  else if (type == 22)
    {
      mFileName += "merged_";
      fileWithoutExtension += "merged_";
    }

  fileWithoutExtension += vtksys::SystemTools::GetFilenameWithoutLastExtension(filenames[0]);
  mFileName += filenames[0];
  mDim = dim;
  mComponent = component;
  if(mConvertor == NULL)
    mConvertor = new vvConvertITKImageToVTKDataFilter;
  mConvertor->SetInputFileName(filenames);
  mConvertor->Update(dim,component.c_str(),type);
  
  //std::cout << fileWithoutExtension << " " << filenames[0] << " dim= " << dim << " c=" << component.c_str() << std::endl;

  if(mConvertor->GetLastError().size() == 0)
    {
      for( unsigned int i = 0; i < mSlicers.size(); i++)
	{
	  mSlicers[i]->SetFileName(fileWithoutExtension);
	  mSlicers[i]->SetImage(mConvertor->GetOutput());
	}
    }
  else
    {
      mLastError = mConvertor->GetLastError();
      return false;
    }
  return true;
}

bool vvSlicersManager::SetOverlay(std::string filename,int dim, std::string component)
{
  //std::cout << "vvSlicerManager::SetOverlay()" << std::endl;
  mOverlayName = filename;
  mOverlayComponent = component;
  if(dim > mDim)
    {
      mLastError = " Overlay dimension cannot be greater then reference image!";
      return false;
    }
  if(mOverlayConvertor == NULL)
    mOverlayConvertor = new vvConvertITKImageToVTKDataFilter;
  std::vector<std::string> filenames;
  filenames.push_back(filename);
  mOverlayConvertor->SetInputFileName(filenames);
  mOverlayConvertor->Update(mDim,component.c_str(),mType);
  if(mOverlayConvertor->GetLastError().size() == 0)
    {
      for( unsigned int i = 0; i < mSlicers.size(); i++)
	{
	  mSlicers[i]->SetOverlay(mOverlayConvertor->GetOutput());
	}
    }
  else
    {
      mLastError = mOverlayConvertor->GetLastError();
      return false;
    }
  return true;
}

bool vvSlicersManager::SetFusion(std::string filename,int dim, std::string component)
{
  //std::cout << "vvSlicerManager::SetFusion()" << std::endl;
  mFusionName = filename;
  mFusionComponent = component;
  if(dim > mDim)
    {
      mLastError = " Overlay dimension cannot be greater then reference image!";
      return false;
    }
  if(mFusionConvertor == NULL)
    mFusionConvertor = new vvConvertITKImageToVTKDataFilter;
  std::vector<std::string> filenames;
  filenames.push_back(filename);
  mFusionConvertor->SetInputFileName(filenames);
  mFusionConvertor->Update(mDim,component.c_str(),mType);
  if(mFusionConvertor->GetLastError().size() == 0)
    {
      for( unsigned int i = 0; i < mSlicers.size(); i++)
	{
	  mSlicers[i]->SetFusion(mFusionConvertor->GetOutput());
	}
    }
  else
    {
    //std::cout << "error" << std::endl;
    mLastError = mFusionConvertor->GetLastError();
    return false;
    }
  double *fusRange = mFusionConvertor->GetOutput()->GetImage()[0]->GetScalarRange();
  mFusionLevel = (fusRange[0]+fusRange[1])/2;
  mFusionWindow = fusRange[1]-fusRange[0];
  return true;
}

bool vvSlicersManager::SetVF(std::string filename,int dim, std::string component)
{
  //std::cout << "vvSlicerManager::SetVF()" << std::endl;
  mVFName = filename;
  mVFComponent = component;
  if(dim > mDim)
    {
      mLastError = " Vector field dimension cannot be greater then reference image!";
      return false;
    }
  if(mVectorConvertor == NULL)
    mVectorConvertor = new vvConvertITKVectorToVTKDataFilter;
  mVectorConvertor->SetInputFileName(filename);
  mVectorConvertor->Update(mDim,component.c_str());
  if(mVectorConvertor->GetLastError().size() == 0)
    {
      for( unsigned int i = 0; i < mSlicers.size(); i++)
	{
	  mSlicers[i]->SetVF(mVectorConvertor->GetOutput());
	}
    }
  else
    {
      mLastError = mVectorConvertor->GetLastError();
      return false;
    }
  return true;
}

void vvSlicersManager::SetExtractedImage(std::string filename,vvImage *image, int dim, std::string component, int slice)
{
  //std::cout << "vvSlicerManager::SetExtractedImage()" << std::endl;
  mFileName = filename;
  mDim = dim - 1;
  mComponent = component;

  if(dim == 4)
    {
      vvImage *extracted = new vvImage;
      extracted->AddImage(image->GetImage()[slice]);
      for( unsigned int i = 0; i < mSlicers.size(); i++)
	{
	  mSlicers[i]->SetFileName(vtksys::SystemTools::GetFilenameWithoutLastExtension(filename));
	  mSlicers[i]->SetImage(extracted);
	}
    }
  else
    {
      vvImage *extracted = new vvImage;
      vtkImageClip* clipper = vtkImageClip::New();
      int extent[6];
      image->GetImage()[0]->GetWholeExtent(extent);
      clipper->SetInput(image->GetImage()[0]);
      clipper->SetOutputWholeExtent(extent[0],extent[1],extent[2],extent[3],slice,slice);
      clipper->Update();
      extracted->AddImage(clipper->GetOutput());
      for( unsigned int i = 0; i < mSlicers.size(); i++)
	{
	  mSlicers[i]->SetFileName(vtksys::SystemTools::GetFilenameWithoutLastExtension(filename));
	  mSlicers[i]->SetImage(extracted);
	}
      clipper->Delete();
    }
}

vvSlicer* vvSlicersManager::GetSlicer(int i)
{
  ////std::cout << "vvSlicerManager::GetSlicer(" << i << ")" << std::endl;
  return mSlicers[i];
}

void vvSlicersManager::UpdateSlicer(int num, int state)
{
  //std::cout << "vvSlicerManager::UpdateSlicer(" << num << "," << state << ")" << std::endl;
  if(mSlicers[num]->GetImage())
    {
      if(state >= 1)
	{
	  mSlicers[num]->SetDisplayMode(1);
	}
      else
	{
	  mSlicers[num]->SetDisplayMode(0);
	}
    }
}

void vvSlicersManager::SetSlicerWindow(int i, vtkRenderWindow* RW)
{
  //std::cout << "vvSlicerManager::SetSlicerWindow(" << i << ")" << std::endl;
  mSlicers[i]->SetRenderWindow(i,RW);
}

void vvSlicersManager::SetInteractorStyleNavigator(int i, vvInteractorStyleNavigator* style)
{
  //std::cout << "vvSlicerManager::SetInteractorStyleNavigator(" << i << ")" << std::endl;
  vvSlicersManagerCallback *smc = vvSlicersManagerCallback::New();
  smc->SM = this;
  mSlicers[i]->GetRenderWindow()->GetInteractor()->SetInteractorStyle(style);

  mSlicers[i]->GetRenderWindow()->GetInteractor()->
    GetInteractorStyle()->AddObserver(vtkCommand::KeyPressEvent, smc);
  mSlicers[i]->GetRenderWindow()->GetInteractor()->
    GetInteractorStyle()->AddObserver(vtkCommand::WindowLevelEvent, smc);
  mSlicers[i]->GetRenderWindow()->GetInteractor()->
    GetInteractorStyle()->AddObserver(vtkCommand::EndWindowLevelEvent, smc);
  mSlicers[i]->GetRenderWindow()->GetInteractor()->
    GetInteractorStyle()->AddObserver(vtkCommand::StartWindowLevelEvent, smc);
  mSlicers[i]->GetRenderWindow()->GetInteractor()->
    GetInteractorStyle()->AddObserver(vtkCommand::PickEvent, smc);
  mSlicers[i]->GetRenderWindow()->GetInteractor()->
    GetInteractorStyle()->AddObserver(vtkCommand::StartPickEvent, smc);
  mSlicers[i]->GetRenderWindow()->GetInteractor()->
    GetInteractorStyle()->AddObserver(vtkCommand::LeaveEvent, smc);
  mSlicers[i]->GetRenderWindow()->GetInteractor()->
    GetInteractorStyle()->AddObserver(vtkCommand::UserEvent, smc);
  mSlicers[i]->GetRenderWindow()->GetInteractor()->
    GetInteractorStyle()->AddObserver(vtkCommand::MouseWheelForwardEvent, smc);
  mSlicers[i]->GetRenderWindow()->GetInteractor()->
    GetInteractorStyle()->AddObserver(vtkCommand::MouseWheelBackwardEvent, smc);
  smc->Delete();
}

void vvSlicersManager::SetTSlice(int slice)
{
  //std::cout << "vvSlicerManager::SetTSlice(" << slice << ")" << std::endl;
  if(slice < 0)
    slice = 0;
  else if (slice > mSlicers[0]->GetTMax())
    slice = mSlicers[0]->GetTMax();
  if(mLandmarks)
    mLandmarks->SetTime(slice);
  for( unsigned int i = 0; i < mSlicers.size(); i++)
    {
      mSlicers[i]->SetTSlice(slice);
      if(mSlicers[i]->GetImageActor()->GetVisibility())
	UpdateTSlice(i);
    }
}

void vvSlicersManager::SetNextTSlice()
{
  //std::cout << "vvSlicerManager::SetNextTSlice()" << std::endl;
  int t = mSlicers[0]->GetTSlice();
  t++;
  if(t >= mSlicers[0]->GetTMax())
    t = 0;
  if(mLandmarks)
    mLandmarks->SetTime(t);
  for( unsigned int i = 0; i < mSlicers.size(); i++)
    {
      mSlicers[i]->SetTSlice(t);
    }
}

void vvSlicersManager::SetPreviousTSlice()
{
  //std::cout << "vvSlicerManager::SetPreviousTSlice()" << std::endl;
  int t = mSlicers[0]->GetTSlice();
  t--;
  if(t < 0)
    t = mSlicers[0]->GetTMax();
  if(mLandmarks)
    mLandmarks->SetTime(t);
  for( unsigned int i = 0; i < mSlicers.size(); i++)
    {
      mSlicers[i]->SetTSlice(t);
    }
}

void vvSlicersManager::SetTSliceInSlicer(int tslice, int slicer)
{
  if(tslice < 0)
    tslice = 0;
  else if (tslice > mSlicers[slicer]->GetTMax())
    tslice = mSlicers[slicer]->GetTMax();
  if(mLandmarks)
    mLandmarks->SetTime(tslice);
  mSlicers[slicer]->SetTSlice(tslice);
  if(mSlicers[slicer]->GetImageActor()->GetVisibility())
    UpdateTSlice(tslice);
}

void vvSlicersManager::SetColorWindow(double s)
{
  //std::cout << "vvSlicerManager::SetColorWindow(" << s << ")" << std::endl;
  for( unsigned int i = 0; i < mSlicers.size(); i++)
    {
      mSlicers[i]->SetColorWindow(s);
    }
}

void vvSlicersManager::SetColorLevel(double s)
{
  //std::cout << "vvSlicerManager::SetColorLevel(" << s << ")" << std::endl;
  for( unsigned int i = 0; i < mSlicers.size(); i++)
    {
      mSlicers[i]->SetColorLevel(s);
    }
}

void vvSlicersManager::SetCursorVisibility(int s)
{
  //std::cout << "vvSlicerManager::SetCursorVisibility(" << s << ")" << std::endl;
  for( unsigned int i = 0; i < mSlicers.size(); i++)
    {
      mSlicers[i]->SetCursorVisibility(s);
    }
}

void vvSlicersManager::SetOpacity(int i, double factor)
{
  //std::cout << "vvSlicerManager::SetOpacity(" << i << "," << factor << ")" << std::endl;
  mSlicers[i]->SetOpacity(1/factor);
}

void vvSlicersManager::ResetWindowLevel()
{
  //std::cout << "vvSlicerManager::ResetWindowLevel()" << std::endl;
  for( unsigned int i = 0; i < mSlicers.size(); i++)
    {
      mSlicers[i]->GetInput()->UpdateInformation();
      mSlicers[i]->GetInput()->SetUpdateExtent
	(mSlicers[i]->GetInput()->GetWholeExtent());
      mSlicers[i]->GetInput()->Update();
      double *range = mSlicers[i]->GetInput()->GetScalarRange();
      mSlicers[i]->SetColorWindow(range[1] - range[0]);
      mSlicers[i]->SetColorLevel(0.5 * (range[1] + range[0]));
    }
}

void vvSlicersManager::UpdateViews(int current,int slicer)
{
  //std::cout << "vvSlicerManager::UpdateViews("<< current << "," << slicer << ")" << std::endl;
  double x = (mSlicers[slicer]->GetCurrentPosition()[0] - mSlicers[slicer]->GetInput()->GetOrigin()[0])
    /mSlicers[slicer]->GetInput()->GetSpacing()[0];
  double y = (mSlicers[slicer]->GetCurrentPosition()[1] - mSlicers[slicer]->GetInput()->GetOrigin()[1])
    /mSlicers[slicer]->GetInput()->GetSpacing()[1];
  double z = (mSlicers[slicer]->GetCurrentPosition()[2] - mSlicers[slicer]->GetInput()->GetOrigin()[2])
    /mSlicers[slicer]->GetInput()->GetSpacing()[2];

  if(x >= mSlicers[slicer]->GetInput()->GetWholeExtent()[0] && 
     x <= mSlicers[slicer]->GetInput()->GetWholeExtent()[1] && 
     y >= mSlicers[slicer]->GetInput()->GetWholeExtent()[2] && 
     y <= mSlicers[slicer]->GetInput()->GetWholeExtent()[3] && 
     z >= mSlicers[slicer]->GetInput()->GetWholeExtent()[4] && 
     z <= mSlicers[slicer]->GetInput()->GetWholeExtent()[5])
    {
      mSlicers[slicer]->UpdateCursorPosition();
      mSlicers[slicer]->SetCursorColor(10,212,255);

      switch (mSlicers[slicer]->GetSliceOrientation())
	{
	case vtkImageViewer2::SLICE_ORIENTATION_XY:
	  if(mSlicers[slicer]->GetSlice() == (int)floor(z))
	    mSlicers[slicer]->Render();
	  else
	    mSlicers[slicer]->SetSlice((int)floor(z));
          break;

	case vtkImageViewer2::SLICE_ORIENTATION_XZ:
	  if(mSlicers[slicer]->GetSlice() == (int)floor(y))
	    mSlicers[slicer]->Render();
	  else
	    mSlicers[slicer]->SetSlice((int)floor(y));
          break;

	case vtkImageViewer2::SLICE_ORIENTATION_YZ:
	  if(mSlicers[slicer]->GetSlice() == (int)floor(x))
	    mSlicers[slicer]->Render();
	  else
	    mSlicers[slicer]->SetSlice((int)floor(x));
          break;
	}

      for( unsigned int i = 0; i < mSlicers.size(); i++)
	{
	  if(i != (unsigned int)slicer && mSlicers[i]->GetImageActor()->GetVisibility() 
	     && mSlicers[i]->GetRenderWindow()->GetSize()[0] > 2
	     && mSlicers[i]->GetRenderWindow()->GetSize()[1] > 2)
	    {
	      mSlicers[i]->SetCurrentPosition(mSlicers[slicer]->GetCurrentPosition()[0],
					      mSlicers[slicer]->GetCurrentPosition()[1],
					      mSlicers[slicer]->GetCurrentPosition()[2],
					      mSlicers[slicer]->GetTSlice());
	      mSlicers[i]->UpdateCursorPosition();
	      if(current) //do not display corner annotation if image is the one picked
		{
		  mSlicers[i]->SetCurrentPosition(-VTK_DOUBLE_MAX,-VTK_DOUBLE_MAX,
						  -VTK_DOUBLE_MAX, mSlicers[slicer]->GetTSlice());
		  mSlicers[i]->SetCursorColor(255,10,212);
		}
	      else
		{
		  mSlicers[i]->SetCursorColor(150,10,282);
		}
	      switch (mSlicers[i]->GetSliceOrientation())
		{
		case vtkImageViewer2::SLICE_ORIENTATION_XY:
		  if(mSlicers[i]->GetSlice() == (int)floor(z))
		    mSlicers[i]->Render();
		  else
		    mSlicers[i]->SetSlice((int)floor(z));
		  break;

		case vtkImageViewer2::SLICE_ORIENTATION_XZ:
		  if(mSlicers[i]->GetSlice() == (int)floor(y))
		    mSlicers[i]->Render();
		  else
		    mSlicers[i]->SetSlice((int)floor(y));
		  break;

		case vtkImageViewer2::SLICE_ORIENTATION_YZ:
		  if(mSlicers[i]->GetSlice() == (int)floor(x))
		    mSlicers[i]->Render();
		  else
		    mSlicers[i]->SetSlice((int)floor(x));
		  break;
		}
	      UpdateSlice(i);
	      UpdateTSlice(i);
	    }
	}
    }
}

void vvSlicersManager::UpdateLinked(int slicer)
{
  //std::cout << "vvSlicerManager::UpdateLinked(" << slicer << ")" << std::endl;
  double x = (mSlicers[slicer]->GetCurrentPosition()[0] - mSlicers[slicer]->GetInput()->GetOrigin()[0])
    /mSlicers[slicer]->GetInput()->GetSpacing()[0];
  double y = (mSlicers[slicer]->GetCurrentPosition()[1] - mSlicers[slicer]->GetInput()->GetOrigin()[1])
    /mSlicers[slicer]->GetInput()->GetSpacing()[1];
  double z = (mSlicers[slicer]->GetCurrentPosition()[2] - mSlicers[slicer]->GetInput()->GetOrigin()[2])
    /mSlicers[slicer]->GetInput()->GetSpacing()[2];

  if(x >= mSlicers[slicer]->GetInput()->GetWholeExtent()[0] && 
     x <= mSlicers[slicer]->GetInput()->GetWholeExtent()[1] && 
     y >= mSlicers[slicer]->GetInput()->GetWholeExtent()[2] && 
     y <= mSlicers[slicer]->GetInput()->GetWholeExtent()[3] && 
     z >= mSlicers[slicer]->GetInput()->GetWholeExtent()[4] && 
     z <= mSlicers[slicer]->GetInput()->GetWholeExtent()[5])
    {
      for(std::list<std::string>::const_iterator i = mLinkedId.begin(); i != mLinkedId.end(); i++)
	{
	  emit UpdateLinkManager(*i, slicer,mSlicers[slicer]->GetCurrentPosition()[0],
				 mSlicers[slicer]->GetCurrentPosition()[1],
				 mSlicers[slicer]->GetCurrentPosition()[2],mSlicers[slicer]->GetTSlice());
	}
    }
}
double vvSlicersManager::GetColorWindow()
{
  //std::cout << "vvSlicerManager::GetColorWindow()" << std::endl;
  if (mSlicers.size())
    return mSlicers[0]->GetColorWindow();
  return -1;
}

double vvSlicersManager::GetColorLevel()
{
  //std::cout << "vvSlicerManager::GetColorLevel()" << std::endl;
  if (mSlicers.size())
    return mSlicers[0]->GetColorLevel();
  return -1;
}

void vvSlicersManager::Render()
{
  //std::cout << "vvSlicerManager::Render()" << std::endl;
  for( unsigned int i = 0; i < mSlicers.size(); i++)
    {
      mSlicers[i]->Render();
    }
}

void vvSlicersManager::GenerateDefaultLookupTable()
{
  //std::cout << "vvSlicerManager::GenerateDefaultLookupTable()" << std::endl;
  SetPreset(mPreset);
  SetColorMap(mColorMap);
}

void vvSlicersManager::Reload()
{
  //std::cout << "vvSlicerManager::Reload()" << std::endl;
  mConvertor->Update(mDim,mComponent.c_str(),mType);
  for( unsigned int i = 0; i < mSlicers.size(); i++)
    {
      mSlicers[i]->SetImage(mConvertor->GetOutput());
    }
}

void vvSlicersManager::ReloadVF()
{
  //std::cout << "vvSlicerManager::ReloadVF()" << std::endl;
  mVectorConvertor->Update(mDim,mVFComponent.c_str());
  for( unsigned int i = 0; i < mSlicers.size(); i++)
    {
      mSlicers[i]->SetVF(mVectorConvertor->GetOutput());
      mSlicers[i]->Render();
    }
}
void vvSlicersManager::RemoveVF()
{
  //std::cout << "vvSlicerManager::RemoveVF()" << std::endl;
  for(unsigned int i = 0; i < mSlicers.size();i++)
    {
      mSlicers[i]->RemoveVF();
    }
}

void vvSlicersManager::RemoveOverlay()
{
  //std::cout << "vvSlicerManager::RemoveOverlay()" << std::endl;
  for(unsigned int i = 0; i < mSlicers.size();i++)
    {
      mSlicers[i]->RemoveOverlay();
    }
}

void vvSlicersManager::RemoveFusion()
{
  //std::cout << "vvSlicerManager::RemoveFusion()" << std::endl;
  for(unsigned int i = 0; i < mSlicers.size();i++)
    {
      mSlicers[i]->RemoveFusion();
    }
}

void vvSlicersManager::RemoveActors()
{
  //std::cout << "vvSlicerManager::RemoveActors()" << std::endl;
  for( unsigned int i = 0; i < mSlicers.size(); i++)
    {
      mSlicers[i]->SetDisplayMode(0);
      mSlicers[i]->GetRenderer()->RemoveActor(mSlicers[i]->GetImageActor());
    }
}

void vvSlicersManager::UpdateInfoOnCursorPosition(int slicer)
{
  //std::cout << "vvSlicerManager::UpdateInfoOnCursorPosition(" << slicer << ")" << std::endl;
  int view = mSlicers[slicer]->GetSliceOrientation();
  int slice = mSlicers[slicer]->GetSlice();
  double x = mSlicers[slicer]->GetCursorPosition()[0];
  double y = mSlicers[slicer]->GetCursorPosition()[1];
  double z = mSlicers[slicer]->GetCursorPosition()[2];
  double X = (x - mSlicers[slicer]->GetInput()->GetOrigin()[0])/mSlicers[slicer]->GetInput()->GetSpacing()[0];
  double Y = (y - mSlicers[slicer]->GetInput()->GetOrigin()[1])/mSlicers[slicer]->GetInput()->GetSpacing()[1];
  double Z = (z - mSlicers[slicer]->GetInput()->GetOrigin()[2])/mSlicers[slicer]->GetInput()->GetSpacing()[2];
  double value = -VTK_DOUBLE_MAX;
  int displayVec = 0;
  double xVec, yVec, zVec, valueVec;
  int displayOver = 0;
  int displayFus = 0;
  double valueOver, valueFus;
  if(X >= mSlicers[slicer]->GetInput()->GetWholeExtent()[0] && 
     X <= mSlicers[slicer]->GetInput()->GetWholeExtent()[1] && 
     Y >= mSlicers[slicer]->GetInput()->GetWholeExtent()[2] && 
     Y <= mSlicers[slicer]->GetInput()->GetWholeExtent()[3] && 
     Z >= mSlicers[slicer]->GetInput()->GetWholeExtent()[4] && 
     Z <= mSlicers[slicer]->GetInput()->GetWholeExtent()[5])
    {
      value = mSlicers[slicer]->GetInput()->GetScalarComponentAsDouble(
		(int)floor(X),
		(int)floor(Y),
		(int)floor(Z),0);
      if(mSlicers[slicer]->GetVFActor() && mSlicers[slicer]->GetVFActor()->GetVisibility())
        {
        displayVec = 1;
        unsigned int currentTime = mSlicers[slicer]->GetTSlice();
        vtkStructuredGrid *vf = NULL;

        if(mSlicers[slicer]->GetVF()->GetImage().size() > currentTime)
          vf = mSlicers[slicer]->GetVF()->GetImage()[currentTime];
        else
          vf = mSlicers[slicer]->GetVF()->GetImage()[0];
        
        int id = -1;
        if(vf)
          id = vf->FindPoint(x,y,z);
        if(id >= 0 && id < vf->GetPointData()->GetVectors()->GetNumberOfTuples())
          {
          xVec = vf->GetPointData()->GetVectors()->GetTuple(id)[0];
          yVec = vf->GetPointData()->GetVectors()->GetTuple(id)[1];
          zVec = vf->GetPointData()->GetVectors()->GetTuple(id)[2];
          valueVec = sqrt(xVec*xVec + yVec*yVec + zVec*zVec);
          }
        }
      if(mSlicers[slicer]->GetOverlayActor() && mSlicers[slicer]->GetOverlayActor()->GetVisibility())
        {
	  displayOver = 1;
	  double Xover = (x - mSlicers[slicer]->GetOverlay()->GetOrigin()[0])
	    /mSlicers[slicer]->GetOverlay()->GetSpacing()[0];
	  double Yover = (y - mSlicers[slicer]->GetOverlay()->GetOrigin()[1])
	    /mSlicers[slicer]->GetOverlay()->GetSpacing()[1];
	  double Zover = (z - mSlicers[slicer]->GetOverlay()->GetOrigin()[2])
	    /mSlicers[slicer]->GetOverlay()->GetSpacing()[2];
	  if(Xover >= mSlicers[slicer]->GetOverlayMapper()->GetInput()->GetWholeExtent()[0] && 
	     Xover <= mSlicers[slicer]->GetOverlayMapper()->GetInput()->GetWholeExtent()[1] && 
	     Yover >= mSlicers[slicer]->GetOverlayMapper()->GetInput()->GetWholeExtent()[2] && 
	     Yover <= mSlicers[slicer]->GetOverlayMapper()->GetInput()->GetWholeExtent()[3] && 
	     Zover >= mSlicers[slicer]->GetOverlayMapper()->GetInput()->GetWholeExtent()[4] && 
	     Zover <= mSlicers[slicer]->GetOverlayMapper()->GetInput()->GetWholeExtent()[5])
	    {
	      valueOver = static_cast<vtkImageData*>(mSlicers[slicer]->GetOverlayMapper()->GetInput())->
		GetScalarComponentAsDouble(
					   (int)floor(Xover),
					   (int)floor(Yover),
					   (int)floor(Zover),0);
	    }
        }
      if(mSlicers[slicer]->GetFusionActor() && mSlicers[slicer]->GetFusionActor()->GetVisibility())
        {
	  displayFus = 1;
	  double Xfus = (x - mSlicers[slicer]->GetFusion()->GetOrigin()[0])
	    /mSlicers[slicer]->GetFusion()->GetSpacing()[0];
	  double Yfus = (y - mSlicers[slicer]->GetFusion()->GetOrigin()[1])
	    /mSlicers[slicer]->GetFusion()->GetSpacing()[1];
	  double Zfus = (z - mSlicers[slicer]->GetFusion()->GetOrigin()[2])
	    /mSlicers[slicer]->GetFusion()->GetSpacing()[2];
	  if(Xfus >= mSlicers[slicer]->GetFusionMapper()->GetInput()->GetWholeExtent()[0] && 
	     Xfus <= mSlicers[slicer]->GetFusionMapper()->GetInput()->GetWholeExtent()[1] && 
	     Yfus >= mSlicers[slicer]->GetFusionMapper()->GetInput()->GetWholeExtent()[2] && 
	     Yfus <= mSlicers[slicer]->GetFusionMapper()->GetInput()->GetWholeExtent()[3] && 
	     Zfus >= mSlicers[slicer]->GetFusionMapper()->GetInput()->GetWholeExtent()[4] && 
	     Zfus <= mSlicers[slicer]->GetFusionMapper()->GetInput()->GetWholeExtent()[5])
	    {
	      valueFus = static_cast<vtkImageData*>(mSlicers[slicer]->GetFusionMapper()->GetInput())->
		GetScalarComponentAsDouble(
					   (int)floor(Xfus),
					   (int)floor(Yfus),
					   (int)floor(Zfus),0);
	    }
        }
    }
  emit UpdatePosition(mSlicers[slicer]->GetCursorVisibility(),
		      x,y,z,X,Y,Z,value);
  emit UpdateVector(displayVec,xVec, yVec, zVec, valueVec);
  emit UpdateOverlay(displayOver,valueOver,value);
  emit UpdateFusion(displayFus,valueFus);
  for(unsigned int i = 0; i < mSlicers.size(); i++)
    {
      if(mSlicers[i]->GetImageActor()->GetVisibility() == 1)
	emit UpdateWindows(i,mSlicers[i]->GetSliceOrientation(),mSlicers[i]->GetSlice());
      else
	emit UpdateWindows(i,-1,-1);
    }
}

void vvSlicersManager::Activated()
{
  emit currentImageChanged(mId);
}

void vvSlicersManager::UpdateWindowLevel()
{
  //std::cout << "vvSlicerManager::UpdateWindowLevel()" << std::endl;
  emit WindowLevelChanged(mSlicers[0]->GetColorWindow(),mSlicers[0]->GetColorLevel(),mPreset,mColorMap);
}

void vvSlicersManager::UpdateSlice(int slicer)
{
  //std::cout << "vvSlicerManager::UpdateSlice(" << slicer << ")" << std::endl;
  emit UpdateSlice(slicer, mSlicers[slicer]->GetSlice());
}

void vvSlicersManager::UpdateTSlice(int slicer)
{
  //std::cout << "vvSlicerManager::UpdateTSlice(" << slicer << ")" << std::endl;
  emit UpdateTSlice(slicer,mSlicers[0]->GetTSlice());
}

void vvSlicersManager::UpdateSliceRange(int slicer)
{
  //std::cout << "vvSlicerManager::UpdateSliceRange(" << slicer << ")" << std::endl;
  emit UpdateSliceRange(slicer,
			mSlicers[slicer]->GetSliceRange()[0], mSlicers[slicer]->GetSliceRange()[1],
			0,mSlicers[slicer]->GetTMax());
}

void vvSlicersManager::SetPreset(int preset)
{
  //std::cout << "vvSlicerManager::SetPreset(" << preset <<")" << std::endl;
  double range[2];
  range[0] = mSlicers[0]->GetInput()->GetScalarRange()[0];
  range[1] = mSlicers[0]->GetInput()->GetScalarRange()[1];
  vtkLookupTable* LUT = static_cast<vtkLookupTable*>(mSlicers[0]->GetWindowLevel()->GetLookupTable());
  double window = mSlicers[0]->GetColorWindow();
  double level = mSlicers[0]->GetColorLevel();

  switch(preset)
    {
    case 0: 
      if(mComponent == "unsigned_char")
        {
	  window = 255;
	  level = 127;
        }
      else if(mComponent == "short")
        {
	  window = 2000;
	  level = 0;
        }
      else
        {
	  window = range[1] - range[0];
	  level = (range[1] + range[0])* 0.5;
        }
      break;
    case 1:
      window = 2000;
      level = 0;
      break;
    case 2:
      window = 350;
      level = 60;
      break;
    case 3:
      window = 1500;
      level = -500;
      break;
    case 4:
      window = 1000;
      level = 500;
      break;
    case 5:
      window = 1;
      level = 0.5;
      break;
    case 6:
      break;
    }
  mPreset = preset;
  this->SetColorWindow(window);
  this->SetColorLevel(level);

  if(LUT)
    {
      SetColorMap(-1);
    }
}

void vvSlicersManager::SetColorMap(int colormap)
{
  //std::cout << "vvSlicerManager::vvSlicerManager::SetColorMap(" << colormap << ")" << std::endl;
  double range[2];
  range[0] = mSlicers[0]->GetInput()->GetScalarRange()[0];
  range[1] = mSlicers[0]->GetInput()->GetScalarRange()[1];

  double window = mSlicers[0]->GetWindowLevel()->GetWindow();
  double level = mSlicers[0]->GetWindowLevel()->GetLevel();

  vtkLookupTable* LUT = static_cast<vtkLookupTable*>(mSlicers[0]->GetWindowLevel()->GetLookupTable());
  switch(colormap)
    {
    case -1:
      if(LUT)
        LUT->SetTableRange(level-fabs(window)/4,level+fabs(window)/4);
      break;
    case 0:
      LUT = NULL;
      break;
    case 1: 
      if(LUT == NULL)
        LUT = vtkLookupTable::New();
      LUT->SetTableRange(level-fabs(window)/4,level+fabs(window)/4);
      LUT->SetValueRange(1,1);
      LUT->SetSaturationRange(1,1);
      LUT->SetHueRange(0,0.18);
      break;
    case 2: 
      if(LUT == NULL)
        LUT = vtkLookupTable::New();
      LUT->SetTableRange(level-fabs(window)/4,level+fabs(window)/4);
      LUT->SetValueRange(1,1);
      LUT->SetSaturationRange(1,1);
      LUT->SetHueRange(0.4,0.80);
      break;
    case 3: 
      if(LUT == NULL)
        LUT = vtkLookupTable::New();
      LUT->SetTableRange(level-fabs(window)/4,level+fabs(window)/4);
      LUT->SetValueRange(1,1);
      LUT->SetSaturationRange(1,1);
      LUT->SetHueRange(0,1);
      break;
    }
  if (LUT)
    LUT->Build();
  vtkLookupTable* fusLUT = NULL;
  if(mSlicers[0]->GetFusion())
    {
      fusLUT = vtkLookupTable::New();
      double fusRange [2];
      fusRange[0] = mFusionLevel - mFusionWindow/2;
      fusRange[1] = mFusionLevel + mFusionWindow/2;
      fusLUT->SetTableRange(fusRange[0],fusRange[1]);
      fusLUT->SetValueRange(1,1);
      fusLUT->SetSaturationRange(1,1);
      if(mFusionColorMap == 1)
	fusLUT->SetHueRange(0,0.18);
      else if(mFusionColorMap == 2)
	fusLUT->SetHueRange(0.4,0.80);
      else if(mFusionColorMap == 3)
	fusLUT->SetHueRange(0,1);
      fusLUT->Build();
      if(mFusionColorMap == 0)
	fusLUT = NULL;
    }
  for( unsigned int i = 0; i < mSlicers.size(); i++) {
    if(mSlicers[i]->GetOverlay() && mSlicers[i]->GetOverlayActor()->GetVisibility()) {
      vtkLookupTable* supLUT = vtkLookupTable::New();
      supLUT->SetTableRange(range[0],range[1]);
      supLUT->SetValueRange(1,1);
      supLUT->SetSaturationRange(1,1);
      supLUT->SetHueRange(double(mOverlayColor)/360,double(mOverlayColor)/360);
      supLUT->Build();
      vtkLookupTable* invLUT = vtkLookupTable::New();
      invLUT->SetTableRange(range[0],range[1]);
      invLUT->SetValueRange(1,1);
      invLUT->SetSaturationRange(1,1);
      invLUT->SetHueRange(double((mOverlayColor+180)%360)/360,double((mOverlayColor+180)%360)/360);
      invLUT->Build();
      mSlicers[i]->GetWindowLevel()->SetLookupTable(supLUT);
      mSlicers[i]->GetOverlayMapper()->SetLookupTable(invLUT);
      invLUT->Delete();
      supLUT->Delete();
    }
    else if(mSlicers[i]->GetOverlay())
      {
	mSlicers[i]->GetWindowLevel()->SetLookupTable(NULL);
      }
    else
      {
	mSlicers[i]->GetWindowLevel()->SetLookupTable(LUT);
      }
    if(mSlicers[i]->GetFusion() && mSlicers[i]->GetFusionActor()->GetVisibility())
      {
	mSlicers[i]->GetFusionActor()->SetOpacity(double(mFusionOpacity)/100);
	mSlicers[i]->GetFusionMapper()->SetLookupTable(fusLUT);
      }
  }
  if(fusLUT)
    fusLUT->Delete();
  if(colormap >= 0)
    mColorMap = colormap;
}

vvLandmarks* vvSlicersManager::GetLandmarks()
{
  //std::cout << "vvSlicersManager::GetLandmarks()" << std::endl;
  if(mLandmarks == NULL)
    {
      mLandmarks = new vvLandmarks(mSlicers[0]->GetTMax()+1);
      for(unsigned int i = 0; i < mSlicers.size(); i++)
	mSlicers[i]->SetLandmarks(mLandmarks);
    }
  return mLandmarks;
}

void vvSlicersManager::AddLandmark(float x,float y,float z,float t)
{
  //std::cout << "vvSlicersManager::AddLandmark()" << std::endl;
  this->GetLandmarks()->AddLandmark(x,y,z,t);
  emit LandmarkAdded();
}
