/*=========================================================================

 Program:   vv
 Module:    $RCSfile: vvVF.cxx,v $
 Language:  C++
 Date:      $Date: 2008/07/04 08:42:43 $
 Version:   $Revision: 1.2 $
 Author :   Pierre Seroul (pierre.seroul@gmail.com)

Copyright (C) 2008
Léon Bérard cancer center http://oncora1.lyon.fnclcc.fr
CREATIS-LRMN http://www.creatis.insa-lyon.fr

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=========================================================================*/
#include "vvVF.h"

#include "vtkStructuredGrid.h"

vvVF::vvVF()
{
  mVtkImages.resize(0);
  mSpacing.clear();
  mSpacing.push_back(1);
  mSpacing.push_back(1);
  mSpacing.push_back(1);
  mSpacing.push_back(1);
  mOrigin.clear();
  mOrigin.push_back(0);
  mOrigin.push_back(0);
  mOrigin.push_back(0);
  mOrigin.push_back(0);
}

vvVF::~vvVF()
{
  for(unsigned int i = 0; i < mVtkImages.size(); i++) {
    if(mVtkImages[i] != NULL)
      mVtkImages[i]->Delete();
    }
}

void vvVF::SetImage(std::vector<vtkStructuredGrid*> images)
{
  mVtkImages.resize(0);
  for(unsigned int i = 0; i < images.size(); i++) {
    mVtkImages.push_back(images[i]);
    }
}

void vvVF::AddImage(vtkStructuredGrid* image)
{
  mVtkImages.push_back(image);
}

void vvVF::Init()
{
  for(unsigned int i = 0; i < mVtkImages.size(); i++) {
    if(mVtkImages[i] != NULL)
      mVtkImages[i]->Delete();
    }
  mVtkImages.resize(0);
}

int vvVF::GetNumberOfDimensions()
{
  if(mVtkImages.size())
    {
    int dimension = 2;
    int extent[6];
    mVtkImages[0]->GetWholeExtent(extent);
    if(extent[5] - extent[4] >= 2)
      dimension++;
    if(mVtkImages.size() > 1)
      dimension++;
  return dimension;
  }
  return 0;
}

void vvVF::SetSpacing(std::vector<double> spacing)
{
  for(int i = 0; i < 3; i++)
    {
    mSpacing[i] = spacing[i];
    }
}

void vvVF::SetOrigin(std::vector<double> origin)
{
  for(int i = 0; i < 3; i++)
    {
    mOrigin[i] = origin[i];
    }
}

std::vector<double> vvVF::GetSpacing()
{
  return mSpacing;
}

std::vector<double> vvVF::GetOrigin()
{
  return mOrigin;
}

std::vector<int> vvVF::GetSize()
{
  std::vector<int> size0;
  int dim = this->GetNumberOfDimensions();
  for(int i = 0; i < dim; i++)
    {
    if(i == 3)
      size0.push_back(mVtkImages.size());
    else
      size0.push_back(mVtkImages[0]->GetDimensions()[i]);
    }
  return size0;
}

unsigned long vvVF::GetActualMemorySize()
{
  unsigned long size = 0;
  for(unsigned int i = 0; i < mVtkImages.size(); i++) {
    size += mVtkImages[i]->GetActualMemorySize();
    }
  return size;
}
