#include "itkImage.h"
#include "itkImageFileReader.h"
#include "itkExtractImageFilter.h"
#include "itkVectorResampleImageFilter.h"
#include "itkImageFileWriter.h"
#include "itkRecursiveGaussianImageFilter.h"
#include "itkBSplineInterpolateImageFunction.h"
#include "itkAffineTransform.h"
#include "itkVectorNearestNeighborInterpolateImageFunction.h"

int main( int argc, char** argv ) 
{
  if(argc < 2)
  {
  std::cerr << "set an input file ! " << std::endl;
  return 0;
  }

  std::string filename = argv[1];

  //Create the reader
  typedef itk::Vector<float,3> InputVectorType;
  typedef itk::Image< InputVectorType, 4 > InputImageType;
  typedef itk::ImageFileReader<InputImageType> ReaderType;
  ReaderType::Pointer reader = ReaderType::New();
  reader->SetFileName(filename.c_str());

  try
    {
    reader->Update(); 
    }
  catch( itk::ExceptionObject & err ) 
    {
    std::cerr << "Error while reading " << filename.c_str() << " " << err << std::endl;
    return 0;
    }

  std::cout << "image read" << std::endl;
  //Filter for extraction
  /*InputImageType::RegionType inputRegion = reader->GetOutput()->GetLargestPossibleRegion();
  InputImageType::SizeType inputSize = inputRegion.GetSize();

  InputImageType::SizeType extractedRegionSize = inputSize;
  InputImageType::RegionType extractedRegion;
  extractedRegionSize[2] = 40;
  extractedRegion.SetSize(extractedRegionSize);

  InputImageType::IndexType start = inputRegion.GetIndex();
  extractedRegion.SetIndex(start);

  typedef itk::ExtractImageFilter<InputImageType,InputImageType> FilterType;
  FilterType::Pointer filter = FilterType::New();
  filter->SetInput(reader->GetOutput());
  filter->SetExtractionRegion(extractedRegion);*/

  //Filter for resampling
  typedef itk::VectorResampleImageFilter<InputImageType,InputImageType> FilterType;
  FilterType::Pointer filter = FilterType::New();

  typedef itk::AffineTransform<double, InputImageType::ImageDimension> TransformType;
  TransformType::Pointer transform =  TransformType::New();
  filter->SetTransform(transform);

  typedef  InputImageType::SizeType SizeType;
  typedef  InputImageType::SpacingType SpacingType;
  typedef  InputImageType::PointType PointType;
  SizeType outputSize;
  SpacingType outputSpacing;
  PointType outputOrigin;

  outputSize[0] = 117.5;
  outputSize[1] = 88;
  outputSize[2] = 70.5;
  outputSize[3] = 10;

  outputSpacing[0] = 4;
  outputSpacing[1] = 4;
  outputSpacing[2] = 4;
  outputSpacing[3] = 1;

  outputOrigin[0] = 0;
  outputOrigin[1] = 0;
  outputOrigin[2] = 0;
  outputOrigin[3] = 0;

  filter->SetSize(outputSize);
  filter->SetOutputSpacing(outputSpacing);
  filter->SetOutputOrigin(outputOrigin);

  /*typedef itk::BSplineInterpolateImageFunction<InputImageType, double> InterpolatorType; 
  InterpolatorType::Pointer interpolator = InterpolatorType::New();
  interpolator->SetSplineOrder(3);
  filter->SetInterpolator(interpolator);*/

  typedef itk::VectorNearestNeighborInterpolateImageFunction<InputImageType, double> InterpolatorType;     
  InterpolatorType::Pointer interpolator = InterpolatorType::New();
  filter->SetInterpolator(interpolator);

  // Build initial Gaussian bluring (if needed)
  typedef itk::RecursiveGaussianImageFilter<InputImageType, InputImageType> GaussianFilterType;
  std::vector<GaussianFilterType::Pointer> gaussianFilters;
  /*for(unsigned int i=0; i<InputImageType::ImageDimension; i++)
    {
    // Create filter
    gaussianFilters.push_back(GaussianFilterType::New());
    // Set options
    gaussianFilters[i]->SetDirection(i);
    gaussianFilters[i]->SetOrder(GaussianFilterType::ZeroOrder);
    gaussianFilters[i]->SetNormalizeAcrossScale(false);
    gaussianFilters[i]->SetSigma(1); // in millimeter !
    // Set input
    if (i==0) gaussianFilters[i]->SetInput(reader->GetOutput());
    else gaussianFilters[i]->SetInput(gaussianFilters[i-1]->GetOutput());
    }
    filter->SetInput(gaussianFilters[InputImageType::ImageDimension-1]->GetOutput());*/
filter->SetInput(reader->GetOutput());

 try { 
    filter->Update();
  }
  catch( itk::ExceptionObject & err ) {
    std::cerr << "Error while filtering " <<  err << std::endl;
    exit(0);
  }

  //Create the writer
  typedef itk::ImageFileWriter<InputImageType> WriterType;
   WriterType::Pointer writer = WriterType::New();
  writer->SetFileName("output.mhd");
  writer->SetInput(filter->GetOutput());
  try 
    {
    writer->Update(); 
    }
  catch( itk::ExceptionObject & err ) {
    std::cerr << "Error while writing output.mhd " << err << std::endl;
    return 0;
  }

  return 0;
}
