#ifndef IMAGERESAMPLEFILTER_CXX
#define IMAGERESAMPLEFILTER_CXX

/*=========================================================================

Program:   vv
Module:    $RCSfile: vvImageResampleFilter.cxx,v $
Language:  C++
Date:      $Date: 2008/06/23 14:34:32 $
Version:   $Revision: 1.1 $
Author :   David Sarrut (david.sarrut@gmail.com)

Copyright (C) 2008
Léon Bérard cancer center http://oncora1.lyon.fnclcc.fr
CREATIS-LRMN http://www.creatis.insa-lyon.fr

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=========================================================================*/

#include "vvImageResampleFilter.h"

//====================================================================
vvImageResampleFilter::vvImageResampleFilter() {
  mApplyGaussianFilterBefore = false;
  mDefaultPixelValue = 0.0;
  mInterpolatorName = "nn";
  mBSplineOrder=3;
  mUseAnObserver = false;
}
//====================================================================

//====================================================================
void vvImageResampleFilter::Update() {

  // Determine dim, pixel type, number of components
  mDim = mInputvvImage->GetNumberOfDimensions();
  mPixelTypeName = mInputvvImage->GetImage()[0]->GetScalarTypeAsString();
  mNbOfComponents = mInputvvImage->GetImage()[0]->GetNumberOfScalarComponents();

  // Switch by dimension
  if (mDim == 2) { Update_WithDim<2>(); return; }
  if (mDim == 3) { Update_WithDim<3>(); return; }
  if (mDim == 4) { Update_WithDim<4>(); return; }

  std::cerr << "Error, dimension of input image is " << mDim << ", but I only work with 3." << std::endl;
  exit(0);
}
//====================================================================

//====================================================================
void vvImageResampleFilter::SetOutputSize(const std::vector<int> & size) {
  mOutputSize.resize(size.size());
  std::copy(size.begin(), size.end(), mOutputSize.begin());
}
//====================================================================

//====================================================================
void vvImageResampleFilter::SetOutputSpacing(const std::vector<double> & spacing) {
  mOutputSpacing.resize(spacing.size());
  std::copy(spacing.begin(), spacing.end(), mOutputSpacing.begin());
}
//====================================================================

//====================================================================
void vvImageResampleFilter::SetInterpolationName(const std::string & inter) {
  mInterpolatorName = inter;
}
//====================================================================

//====================================================================
void vvImageResampleFilter::SetGaussianSigma(const std::vector<double> & sigma) {
  mApplyGaussianFilterBefore = true;
  mSigma.resize(sigma.size());
  std::copy(sigma.begin(), sigma.end(), mSigma.begin());
}
//====================================================================

//====================================================================
// std::vector<vtkImageData*> & vvImageResampleFilter::GetOutput() {
//   return mOutput;
// }
//====================================================================

#endif

