#ifndef VVIMAGERESAMPLEFILTER_TXX
#define VVIMAGERESAMPLEFILTER_TXX

/*=========================================================================

Program:   vv
Module:    $RCSfile: vvImageResampleFilter.txx,v $
Language:  C++
Date:      $Date: 2008/06/23 14:34:32 $
Version:   $Revision: 1.1 $
Author :   David Sarrut (david.sarrut@gmail.com)

Copyright (C) 2008
Léon Bérard cancer center http://oncora1.lyon.fnclcc.fr
CREATIS-LRMN http://www.creatis.insa-lyon.fr

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=========================================================================*/

//====================================================================
template<unsigned int Dim>
void vvImageResampleFilter::Update_WithDim() { 
/*#define TRY_TYPE(TYPE)							\
  //if (clitk::IsSameType<TYPE>(mPixelTypeName)) { Update_WithDimAndPixelType<Dim, TYPE>(); return; } 
  TRY_TYPE(signed char);
  TRY_TYPE(uchar);
  TRY_TYPE(short);
  TRY_TYPE(ushort);
  TRY_TYPE(int); // no uint ...
  TRY_TYPE(float);
  //TRY_TYPE(double);
#undef TRY_TYPE

  std::string list = clitk::CreateListOfTypes<char, clitk::uchar, short, ushort, int, float, double>();
  std::cerr << "Error, I don't know the type '" << mPixelTypeName << "' for the input image. "
	    << std::endl << "Known types are " << list << std::endl;
  exit(0);*/
}
//====================================================================

//====================================================================
template<unsigned int Dim, class PixelType>
void vvImageResampleFilter::Update_WithDimAndPixelType() {

  typedef itk::Image<PixelType,Dim> ImageType;
  typedef itk::Image<PixelType, Dim>    ConnectorImageType; //DS to change if D=4
  typedef itk::VTKImageToImageFilter <ConnectorImageType> ConnectorType;
  //typedef itk::JoinSeriesImageFilter <ConnectorImageType,OutputImageType> FilterType;
  
  // Conversion to ITK
  //typename FilterType::Pointer filter = FilterType::New();
  //for(int i = 0; i < mInputvvImage->GetSize()[3]; i++) {
  //DD(i);
  DD("Conversion filter");
  typename ConnectorType::Pointer connector = ConnectorType::New();
  connector->SetInput(mInputvvImage->GetImage()[0]);

  if (mUseAnObserver) {
    connector->AddObserver(itk::ProgressEvent(), mObserver);
  }

  connector->Update();
  
  //  typename ImageType::Pointer in = connector->GetOutput();
  DD("Compute filter");
  typename ImageType::Pointer output = ComputeImage<ImageType>(connector->GetOutput());

  // Write //DS to change !
  // typedef itk::ImageFileWriter<ImageType> WriterType;
//   typename WriterType::Pointer writer = WriterType::New();
//   writer->SetFileName("toto.mhd");
//   writer->SetInput(output);
//   writer->Update();

  // Set image to main vvSlicersManager
  mOutputSlicersManager = new vvSlicersManager(4);
  std::string filename = "toto.mhd";
  mOutputSlicersManager->SetImageFromITK<ImageType>(filename, output);
}
//====================================================================

//====================================================================
template<class ImageType>
typename ImageType::Pointer 
vvImageResampleFilter::ComputeImage(const ImageType * inputImage) {

  // Warning
  if (!std::numeric_limits<typename ImageType::PixelType>::is_signed) {
    if ((mInterpolatorName == "bspline") || (mInterpolatorName == "blut")) {
      std::cerr << "Warning : input pixel type is not signed, use bspline interpolation at your own risk ..." << std::endl;
    }
  }

  // Check options
  static unsigned int dim = ImageType::ImageDimension;
  if (mOutputSize.size() != dim) {
    std::cerr << "Please set size with " << dim << " dimensions." << std::endl;
    return NULL;
  }
  if (mOutputSpacing.size() != dim) {
    std::cerr << "Please set spacing with " << dim << " dimensions." << std::endl;
    return NULL;
  }
  mOutputOrigin.resize(dim);

  if (mApplyGaussianFilterBefore && mSigma.size() != dim) {
    std::cerr << "Please set sigma with " << dim << " dimensions." << std::endl;
    return NULL;
  }

  // Some typedefs
  typedef typename ImageType::SizeType SizeType;
  typedef typename ImageType::SpacingType SpacingType;
  typedef typename ImageType::PointType PointType;

  // Create Image Filter
  typedef itk::ResampleImageFilter<ImageType,ImageType> FilterType;
  typename FilterType::Pointer filter = FilterType::New();

  // Instance of the transform object to be passed to the resample
  // filter. By default, identity transform is applied
  typedef itk::AffineTransform<double, ImageType::ImageDimension> TransformType;
  typename TransformType::Pointer transform =  TransformType::New();
  filter->SetTransform(transform);

  // Set filter's parameters
  SizeType outputSize;
  SpacingType outputSpacing;
  PointType outputOrigin;
  for(unsigned int i=0; i<ImageType::ImageDimension; i++) {
    outputSize[i] = mOutputSize[i];
    outputSpacing[i] = mOutputSpacing[i];
    outputOrigin[i] = mOutputOrigin[i];
  }
  filter->SetSize(outputSize);
  filter->SetOutputSpacing(outputSpacing);
  filter->SetOutputOrigin(outputOrigin);
  filter->SetDefaultPixelValue(mDefaultPixelValue);

  // Select interpolator
  if (mInterpolatorName == "NN") {
    typedef itk::NearestNeighborInterpolateImageFunction<ImageType, double> InterpolatorType;     
    typename InterpolatorType::Pointer interpolator = InterpolatorType::New();
    filter->SetInterpolator(interpolator);
  }
  else { 
    if (mInterpolatorName == "Linear") {
      typedef itk::LinearInterpolateImageFunction<ImageType, double> InterpolatorType;     
      typename InterpolatorType::Pointer interpolator =  InterpolatorType::New();
      filter->SetInterpolator(interpolator);
    }
    else {
      if (mInterpolatorName == "BSpline") {
	typedef itk::BSplineInterpolateImageFunction<ImageType, double> InterpolatorType; 
	typename InterpolatorType::Pointer interpolator = InterpolatorType::New();
	interpolator->SetSplineOrder(mBSplineOrder);
	filter->SetInterpolator(interpolator);
      }
      else {
	if (mInterpolatorName == "Blut") {
	  typedef itk::BSplineInterpolateImageFunctionWithLUT<ImageType, double> InterpolatorType; 
	  typename InterpolatorType::Pointer interpolator = InterpolatorType::New();
	  interpolator->SetSplineOrder(mBSplineOrder);
	  interpolator->SetLUTSamplingFactor(mSamplingFactors[0]);
	  interpolator->SetOutputSpacing(outputSpacing);
	  filter->SetInterpolator(interpolator);
	}
	else {
	  std::cerr << "Sorry, I do not know the interpolator '" << mInterpolatorName 
		    << "'. Known interpolators are :  nn, linear, bspline, blut" << std::endl;
	  exit(0);
	}
      }
    }
  }

  // Build initial Gaussian bluring (if needed)
  typedef itk::RecursiveGaussianImageFilter<ImageType, ImageType> GaussianFilterType;
  std::vector<typename GaussianFilterType::Pointer> gaussianFilters;
  if (mApplyGaussianFilterBefore) {
    for(unsigned int i=0; i<ImageType::ImageDimension; i++) {
      // Create filter
      gaussianFilters.push_back(GaussianFilterType::New());
      // Set options
      gaussianFilters[i]->SetDirection(i);
      gaussianFilters[i]->SetOrder(GaussianFilterType::ZeroOrder);
      gaussianFilters[i]->SetNormalizeAcrossScale(false);
      gaussianFilters[i]->SetSigma(mSigma[i]); // in millimeter !
      // Set input
      if (i==0) gaussianFilters[i]->SetInput(inputImage);
      else gaussianFilters[i]->SetInput(gaussianFilters[i-1]->GetOutput());
    }
    filter->SetInput(gaussianFilters[ImageType::ImageDimension-1]->GetOutput());
  }
  else {
    filter->SetInput(inputImage);
  }

  // Observer to display a progress bar
  if (mUseAnObserver) {
    filter->AddObserver(itk::ProgressEvent(), mObserver);
  }

  // Go !
  try { 
    filter->Update();
  }
  catch( itk::ExceptionObject & err ) {
    std::cerr << "Error while filtering " << err << std::endl;
    exit(0);
  }

  // Return result
  return filter->GetOutput();
}
//====================================================================

#endif /* end #define VVIMAGERESAMPLEFILTER_TXX */

