/*=========================================================================

 Program:   vv
 Module:    $RCSfile: vvs.cxx,v $
 Language:  C++
 Date:      $Date: 2008/05/28 14:19:14 $
 Version:   $Revision: 1.3 $
 Author :   Pierre Seroul (pierre.seroul@gmail.com)

Copyright (C) 2008
Léon Bérard cancer center http://oncora1.lyon.fnclcc.fr
CREATIS-LRMN http://www.creatis.insa-lyon.fr

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=========================================================================*/
#include <iostream.h>

#include "itkImageIOBase.h"
#include "itkImageIOFactory.h"

#include "vvVoxImageIO.h"
#include "vvVoxImageIOFactory.h"
#include "vvVfImageIO.h"
#include "vvVfImageIOFactory.h"
#include "vvConvertITKImageToVTKDataFilter.h"
#include "vvSlicersManager.h"
#include "vvInteractorStyleNavigator.h"

#include "vtkRenderWindow.h"
#include "vtkRenderWindowInteractor.h"
#include "vtkRenderer.h"
#include "vtkCornerAnnotation.h"


int main( int argc, char** argv ) 
{
  itk::ImageIOFactory::RegisterBuiltInFactories();
  vv::VoxImageIOFactory::RegisterOneFactory();
  vv::VfImageIOFactory::RegisterOneFactory();

  std::string filename;
  std::string VFname;
  if(argc < 2)
    {
    std::cout << "set a filename" << std::endl;
    return 0;
    }
  else
    {
    filename = argv[1];
    }
  if(argc < 3)
    {
    std::cout << "set a VF filename" << std::endl;
    return 0;
    }
  else
    {
    VFname = argv[2];
    }
  itk::ImageIOBase::Pointer reader = itk::ImageIOFactory::CreateImageIO(
			filename.c_str(), itk::ImageIOFactory::ReadMode);
  if (!reader)
    {
    std::string error = "Cannot read file ";
    error += filename;
    std::cout << error << std::endl;
    return 0;
    }

  reader->SetFileName(filename);
  reader->ReadImageInformation();

  // create the slicers manager with only one slicer
  vvSlicersManager* imageManager = new vvSlicersManager(1);
  std::string component = reader->GetComponentTypeAsString(reader->GetComponentType());
  imageManager->SetImage(filename,reader->GetNumberOfDimensions(), component);

  // add the VF to the current slicer
  itk::ImageIOBase::Pointer VFreader = itk::ImageIOFactory::CreateImageIO(
			VFname.c_str(), itk::ImageIOFactory::ReadMode);
  if (!VFreader)
    {
    std::string error = "Cannot read file ";
    error += VFname;
    std::cout << error << std::endl;
    return 0;
    }

  VFreader->SetFileName(VFname);
  VFreader->ReadImageInformation();

  component = VFreader->GetComponentTypeAsString(VFreader->GetComponentType());
  imageManager->SetVF(VFname,VFreader->GetNumberOfDimensions(), component);

  std::cout << "RWI" << std::endl;

  // create the render window interactor
  vtkRenderWindowInteractor *interactor = vtkRenderWindowInteractor::New();
  imageManager->GetSlicer(0)->SetupInteractor(interactor);
  imageManager->GetVFSlicer(0)->SetRenderWindow(imageManager->GetSlicer(0)->GetRenderWindow());
  imageManager->GetVFSlicer(0)->SetupInteractor(interactor);

  // create the interactor style
  vvInteractorStyleNavigator* style = vvInteractorStyleNavigator::New();
  imageManager->SetInteractorStyleNavigator(0,style);

  // init annotation
  vtkCornerAnnotation *ca = imageManager->GetSlicer(0)->GetAnnotation();
  ca->SetImageActor(imageManager->GetSlicer(0)->GetImageActor());
  ca->SetWindowLevel(imageManager->GetSlicer(0)->GetWindowLevel());
  ca->SetText(2, "<slice>");
  ca->SetText(3, "<window>\n<level>");
  imageManager->GetSlicer(0)->GetRenderer()->AddActor(ca);

  //render
  imageManager->GetSlicer(0)->Render();
  interactor->Initialize();
  interactor->Start();

  //clear all
  delete imageManager;
  style->Delete();
  interactor->Delete();
  //delete convertor;
  return 0;
}
